@@ -7,8 +7,9 @@
  */
 
 #include "SettingsBase.h"
-#include "../core/kcmmetadatahelpers.h"
-#include "sidebar/SidebarMode.h"
+#include "ModuleView.h"
+#include "SidebarMode.h"
+#include "kcmmetadatahelpers.h"
 
 #include <QFileInfo>
 #include <QFontDatabase>
@@ -25,29 +26,32 @@
 #include <KConfigGroup>
 #include <KDesktopFile>
 #include <KFileUtils>
+#include <KHelpMenu>
 #include <KIO/JobUiDelegateFactory>
 #include <KIO/OpenUrlJob>
 #include <KLocalizedString>
 #include <KStandardAction>
-#include <KToolBar>
 #include <KXMLGUIFactory>
 
-#include "BaseData.h"
-#include "ModuleView.h"
-
-SettingsBase::SettingsBase(BaseMode::ApplicationMode mode, const QString &startupModule, const QStringList &startupModuleArgs, QWidget *parent)
-    : KXmlGuiWindow(parent)
+SettingsBase::SettingsBase(SidebarMode::ApplicationMode mode, const QString &startupModule, const QStringList &startupModuleArgs, QWidget *parent)
+    : KMainWindow(parent)
     , m_mode(mode)
     , m_startupModule(startupModule)
     , m_startupModuleArgs(startupModuleArgs)
+    , m_actionCollection(new KActionCollection(this))
+    , m_helpMenu(new KHelpMenu(this))
 {
     // Prepare the view area
     stackedWidget = new QStackedWidget(this);
     setCentralWidget(stackedWidget);
 
+    m_actionCollection->addAssociatedWidget(this);
+
     setProperty("_breeze_no_separator", true);
 
-    if (m_mode == BaseMode::InfoCenter) {
+    setAutoSaveSettings();
+
+    if (m_mode == SidebarMode::InfoCenter) {
         setWindowTitle(i18nd("systemsettings", "Info Center"));
         setWindowIcon(QIcon::fromTheme(QStringLiteral("hwinfo")));
     } else {
@@ -55,8 +59,6 @@ SettingsBase::SettingsBase(BaseMode::ApplicationMode mode, const QString &startu
         setWindowIcon(QIcon::fromTheme(QStringLiteral("preferences-system")));
     }
 
-    spacerWidget = new QWidget(this);
-    spacerWidget->setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Maximum);
     // Initialise the window so we don't flicker
     initToolBar();
     // We can now launch the delayed loading safely
@@ -82,7 +84,7 @@ QSize SettingsBase::sizeHint() const
 void SettingsBase::initApplication()
 {
     // Prepare the menu of all modules
-    auto source = m_mode == BaseMode::InfoCenter ? MetaDataSource::KInfoCenter : MetaDataSource::SystemSettings;
+    auto source = m_mode == SidebarMode::InfoCenter ? MetaDataSource::KInfoCenter : MetaDataSource::SystemSettings;
     pluginModules = findKCMsMetaData(source) << findExternalKCMModules(source);
 
     const QStringList dirs = QStandardPaths::locateAll(QStandardPaths::AppDataLocation, QStringLiteral("categories"), QStandardPaths::LocateDirectory);
@@ -100,9 +102,6 @@ void SettingsBase::initApplication()
         }
     }
 
-    // Prepare the Base Data
-    BaseData::instance()->setMenuItem(rootModule);
-    BaseData::instance()->setHomeItem(homeModule);
     loadCurrentView();
 
     // enforce minimum window size
@@ -126,10 +125,10 @@ void SettingsBase::initToolBar()
 {
     // Fill the toolbar with default actions
     // Exit is the very last action
-    quitAction = actionCollection()->addAction(KStandardAction::Quit, QStringLiteral("quit_action"), this, &QWidget::close);
+    quitAction = m_actionCollection->addAction(KStandardAction::Quit, QStringLiteral("quit_action"), this, &QWidget::close);
 
-    if (m_mode == BaseMode::SystemSettings) {
-        highlightChangesAction = actionCollection()->addAction(QStringLiteral("highlight_changes"), this, [this] {
+    if (m_mode == SidebarMode::SystemSettings) {
+        highlightChangesAction = m_actionCollection->addAction(QStringLiteral("highlight_changes"), this, [this] {
             view->toggleDefaultsIndicatorsVisibility();
         });
         highlightChangesAction->setCheckable(true);
@@ -137,7 +136,7 @@ void SettingsBase::initToolBar()
         highlightChangesAction->setIcon(QIcon::fromTheme(QStringLiteral("draw-highlight")));
     }
 
-    reportPageSpecificBugAction = actionCollection()->addAction(QStringLiteral("report_bug_in_current_module"), this, [=] {
+    reportPageSpecificBugAction = m_actionCollection->addAction(QStringLiteral("report_bug_in_current_module"), this, [=] {
         const QString bugReportUrlString =
             view->moduleView()->activeModuleMetadata().bugReportUrl() + QStringLiteral("&version=") + QGuiApplication::applicationVersion();
         auto job = new KIO::OpenUrlJob(QUrl(bugReportUrlString));
@@ -147,30 +146,10 @@ void SettingsBase::initToolBar()
     reportPageSpecificBugAction->setText(i18nd("systemsettings", "Report a Bug in the Current Page…"));
     reportPageSpecificBugAction->setIcon(QIcon::fromTheme(QStringLiteral("tools-report-bug")));
 
-    // Help after it
-    initHelpMenu();
-
-    // Then a spacer so the search line-edit is kept separate
-    spacerAction = new QWidgetAction(this);
-    spacerAction->setDefaultWidget(spacerWidget);
-    actionCollection()->addAction(QStringLiteral("spacer"), spacerAction);
-    // Initialise the Window
-    setupGUI(Save | Create, QString());
-    menuBar()->hide();
-
-    // Toolbar & Configuration
-    helpActionMenu->setMenu(dynamic_cast<QMenu *>(factory()->container(QStringLiteral("help"), this)));
-    toolBar()->setMovable(false); // We don't allow any changes
-    changeToolBar(BaseMode::Search | BaseMode::Configure);
-}
-
-void SettingsBase::initHelpMenu()
-{
-    helpActionMenu = new KActionMenu(QIcon::fromTheme(QStringLiteral("help-contents")), i18nd("systemsettings", "Help"), this);
-    helpActionMenu->setPopupMode(QToolButton::InstantPopup);
-    actionCollection()->addAction(QStringLiteral("help_toolbar_menu"), helpActionMenu);
-    // Add the custom actions
-    aboutViewAction = actionCollection()->addAction(KStandardAction::AboutApp, QStringLiteral("help_about_view"), this, &SettingsBase::about);
+    m_actionCollection->addAction(QStringLiteral("help_report_bug"), m_helpMenu->action(KHelpMenu::menuReportBug));
+    m_actionCollection->addAction(QStringLiteral("help_contents"), m_helpMenu->action(KHelpMenu::menuHelpContents));
+    m_actionCollection->addAction(QStringLiteral("help_about_app"), m_helpMenu->action(KHelpMenu::menuAboutApp));
+    m_actionCollection->addAction(QStringLiteral("help_about_kde"), m_helpMenu->action(KHelpMenu::menuAboutKDE));
 }
 
 void SettingsBase::initMenuList(MenuItem *parent)
@@ -181,7 +160,7 @@ void SettingsBase::initMenuList(MenuItem *parent)
         const KConfigGroup entry = file.desktopGroup();
         QString parentCategory;
         QString parentCategory2;
-        if (m_mode == BaseMode::InfoCenter) {
+        if (m_mode == SidebarMode::InfoCenter) {
             parentCategory = entry.readEntry("X-KDE-KInfoCenter-Parent-Category");
         } else {
             parentCategory = entry.readEntry("X-KDE-System-Settings-Parent-Category");
@@ -205,7 +184,7 @@ void SettingsBase::initMenuList(MenuItem *parent)
     for (const auto &metaData : std::as_const(pluginModules)) {
         QString category;
         QString categoryv2;
-        if (m_mode == BaseMode::InfoCenter) {
+        if (m_mode == SidebarMode::InfoCenter) {
             category = metaData.value(QStringLiteral("X-KDE-KInfoCenter-Category"));
         } else {
             category = metaData.value(QStringLiteral("X-KDE-System-Settings-Parent-Category"));
@@ -226,9 +205,9 @@ void SettingsBase::initMenuList(MenuItem *parent)
                 infoItem->setMetaData(metaData);
                 infoItem->setCategoryOwner(isCategoryOwner);
 
-                if (m_mode == BaseMode::InfoCenter && metaData.pluginId() == QStringLiteral("kcm_about-distro")) {
+                if (m_mode == SidebarMode::InfoCenter && metaData.pluginId() == QStringLiteral("kcm_about-distro")) {
                     homeModule = infoItem;
-                } else if (m_mode == BaseMode::SystemSettings && metaData.pluginId() == QStringLiteral("kcm_landingpage")) {
+                } else if (m_mode == SidebarMode::SystemSettings && metaData.pluginId() == QStringLiteral("kcm_landingpage")) {
                     homeModule = infoItem;
                 }
             }
@@ -241,7 +220,6 @@ void SettingsBase::initMenuList(MenuItem *parent)
 bool SettingsBase::queryClose()
 {
     bool changes = true;
-    view->saveState();
     changes = view->moduleView()->resolveChanges();
     return changes;
 }
@@ -275,24 +253,13 @@ void SettingsBase::about()
 
 void SettingsBase::loadCurrentView()
 {
-    view = new SidebarMode(this, {m_mode, m_startupModule, m_startupModuleArgs});
-    connect(view, &BaseMode::changeToolBarItems, this, &SettingsBase::changeToolBar);
-    connect(view, &BaseMode::actionsChanged, this, &SettingsBase::updateViewActions);
-    connect(view, &BaseMode::viewChanged, this, &SettingsBase::viewChange);
-    view->saveState();
+    view = new SidebarMode(this, m_mode, m_startupModule, m_startupModuleArgs, m_actionCollection, homeModule, rootModule);
+    connect(view, &SidebarMode::viewChanged, this, &SettingsBase::viewChange);
 
     if (stackedWidget->indexOf(view->mainWidget()) == -1) {
         stackedWidget->addWidget(view->mainWidget());
     }
 
-    // Handle the tooltips
-    qDeleteAll(tooltipManagers);
-    tooltipManagers.clear();
-    const QList<QAbstractItemView *> theViews = view->views();
-    for (QAbstractItemView *view : theViews) {
-        tooltipManagers << new ToolTipManager(view);
-    }
-
     if (highlightChangesAction) {
         highlightChangesAction->setChecked(view->defaultsIndicatorsVisible());
     }
@@ -300,13 +267,12 @@ void SettingsBase::loadCurrentView()
     viewChange(false);
 
     stackedWidget->setCurrentWidget(view->mainWidget());
-    updateViewActions();
 
     view->giveFocus();
 
     // Update visibility of the "report a bug on this page" and "report bug in general"
     // actions based on whether the current page has a bug report URL set
-    auto reportGeneralBugAction = actionCollection()->action(QStringLiteral("help_report_bug"));
+    auto reportGeneralBugAction = m_actionCollection->action(QStringLiteral("help_report_bug"));
     if (reportGeneralBugAction) {
         reportGeneralBugAction->setVisible(false);
     }
@@ -326,34 +292,6 @@ void SettingsBase::viewChange(bool state)
     setCaption(view->moduleView()->activeModuleName(), state);
 }
 
-void SettingsBase::updateViewActions()
-{
-    guiFactory()->unplugActionList(this, QStringLiteral("viewActions"));
-    guiFactory()->plugActionList(this, QStringLiteral("viewActions"), view->actionsList());
-}
-
-void SettingsBase::changeToolBar(BaseMode::ToolBarItems toolbar)
-{
-    if (sender() != view) {
-        return;
-    }
-    guiFactory()->unplugActionList(this, QStringLiteral("configure"));
-    guiFactory()->unplugActionList(this, QStringLiteral("search"));
-    guiFactory()->unplugActionList(this, QStringLiteral("quit"));
-    if (BaseMode::Search & toolbar) {
-        QList<QAction *> searchBarActions;
-        searchBarActions << spacerAction;
-        guiFactory()->plugActionList(this, QStringLiteral("search"), searchBarActions);
-    }
-    if (BaseMode::Quit & toolbar) {
-        QList<QAction *> quitBarActions;
-        quitBarActions << quitAction;
-        guiFactory()->plugActionList(this, QStringLiteral("quit"), quitBarActions);
-    }
-
-    toolBar()->setVisible(toolbar != BaseMode::NoItems || (view->actionsList().count() > 0));
-}
-
 void SettingsBase::slotGeometryChanged()
 {
     setMinimumSize(SettingsBase::sizeHint());
